""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
CONFIG_FILES_PROPERTIES = ['/opt/cyops/configs/rabbitmq/rabbitmq_users.conf']
CONFIG_FILES_YAML = ['/opt/cyops/configs/cyops-search/config.yml', '/opt/cyops/configs/database/db_config.yml']
CONFIG_FILES_INI = []

with open('/opt/cyops/configs/cyops-search/APPLIANCE_PUBLIC_KEY', 'r') as public_key_file:
    APPLIANCE_PUBLIC_KEY = public_key_file.read().strip()

with open('/opt/cyops/configs/cyops-search/APPLIANCE_PRIVATE_KEY', 'r') as private_key_file:
    APPLIANCE_PRIVATE_KEY = private_key_file.read().strip()

CH_TO_ES_MAPPING = {
    'text': {'type': 'text'},
    'keyword': {'type': 'keyword'},
    'phone': {"type": 'keyword'},
    'email': {"type": 'keyword'},
    'ipv4': {"type": 'keyword'},
    'ipv6': {"type": 'keyword'},
    'url': {"type": 'keyword'},
    'filehash': {"type": 'keyword'},
    'decimal': {"type": 'float'},
    'richtext': {"type": 'text', "analyzer": "richtext_analyzer"},
    'textarea': {"type": "text"},
    'integer': {"type": 'integer'},
    'checkbox': {"type": 'boolean'},
    'object': {"type": 'text'},
    'datetime': {"type": "date", "format": "yyyy-MM-dd HH:mm:ss||epoch_millis"},
    'picklist': {"type": 'keyword'},
    'lookup': {"type": 'keyword'},
    'manyToMany': {"type": 'keyword'}
}

EXCLUDE_MIME_TYPE = ["image", "audio", "video", "application/x-gzip"]

PUBLISH_CREATE = 'publish_create'
PUBLISH_MODIFY = 'publish_modify'
PUBLISH_DELETE = 'publish_delete'
PUBLISH_STATUS = 'publish_status'

BATCH_SIZE = 50

FRAGMENT_SIZE = 50
NUMBER_OF_FRAGMENT = 3

FILE_INDEX = 'files_prod'

MAPPING_SETTINGS = {
    'analysis': {
        "analyzer": {
            "default": {
                "type": "standard",
                "filter": ["lowercase"]
            },
            "ngram_analyzer": {
                "type": "custom",
                "tokenizer": "my_tokenizer",
                "filter": ["lowercase"]
            },
            "richtext_analyzer": {
                "type" : "custom",
                "tokenizer": "standard",
                "filter" : ["lowercase"],
                "char_filter": ["html_strip", "exclude_words_filter"]
            }
        },
        "tokenizer": {
            "my_tokenizer": {
                "type": "ngram",
                "min_gram": 4,
                "max_gram": 4,
                "token_chars": ["letter", "digit", "symbol", "punctuation"]
            }
        },
        "char_filter": {
            "exclude_words_filter": {
                "type": "mapping",
                "mappings_path": "/opt/cyops-search/exclude.list"
            }
        }
    }
}

WORKFLOW_INDEX = 'cyops_workflows'
RULE_INDEX = 'cyops_rules'

IGNORE_MODULES = ['people']


IGNORE_REVERSE_INDEXING_FOR_FORM_TYPE = ['multiselectpicklist']

BUILTIN_MODULE = ['workflows', 'workflow_collections', 'rules']

WORKFLOW_FIELDS = ["name", "description", "id", "@id", "@type", "recordTags", "owners",
                   "modifyDate", "createDate", "deletedAt"]

WORKFLOW_MAPPING = {key: CH_TO_ES_MAPPING['keyword'] for key in WORKFLOW_FIELDS}
WORKFLOW_MAPPING['name'] = CH_TO_ES_MAPPING['text']
WORKFLOW_MAPPING['description'] = CH_TO_ES_MAPPING['text']
WORKFLOW_MAPPING['modifyDate'] = CH_TO_ES_MAPPING['datetime']
WORKFLOW_MAPPING['createDate'] = CH_TO_ES_MAPPING['datetime']
WORKFLOW_MAPPING['deletedAt'] = CH_TO_ES_MAPPING['datetime']

FILES_MAPPING = {'owners': CH_TO_ES_MAPPING['keyword']}

RULE_FIELDS = ["name", "id", "@id", "@type", "recordTags", "tag"]

RULE_MAPPING = {key: CH_TO_ES_MAPPING['keyword'] for key in RULE_FIELDS}
RULE_MAPPING['modifyDate'] = CH_TO_ES_MAPPING['datetime']
RULE_MAPPING['createDate'] = CH_TO_ES_MAPPING['datetime']

WORKFLOW_SETTINGS = {"settings": MAPPING_SETTINGS, "mappings": {"properties": WORKFLOW_MAPPING}}
RULE_SETTINGS = {"settings": MAPPING_SETTINGS, "mappings": {"properties": RULE_MAPPING}}
FILES_SETTINGS = {"settings": MAPPING_SETTINGS, "mappings": {"properties": FILES_MAPPING}}



PREFIX_INDEX = 'cyops_'

API_PREFIX = '/api/3/'

SOURCE_FIELDS = ['@id', '@type', 'displayName', 'createDate', 'modifyDate', 'uuid', 'id', 'filename', 'mimeType']

DISPLAY_NAME_REGEX = "\\{\\{([^{}]+)\\}\\}"

MAX_RETRY_REQUESTS_COUNT = 5

CONTEXT_CODE = {
    1: 200,
    2: 202,
    3: 204
}
