""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import time
from helpers import filter
from util.config import config

KEYWORD_FIELDS = ['indicators']

similarity_settings = config.get('similarity') if config.get('similarity') else {}


def _create_mlt_part(fields, index, id=None, doc=None):
    mlt_queries = []
    for field in fields:
        field_name = field['name']
        field_boost = field.get('weight', 1)
        like = []
        if id:
            like.append({"_index": index, "_id": id})
        if doc:
            like.append({"_index": index, "doc": doc})

        mlt_query = {
            "more_like_this": {
                "fields": [field_name],
                "like": like,
                "min_term_freq": similarity_settings.get('min_term_freq', 1),
                "min_doc_freq": similarity_settings.get('min_doc_freq', 2),
                "max_query_terms": similarity_settings.get('max_query_terms', 25),
                "minimum_should_match": similarity_settings.get('minimum_should_match', "10%"),
                "boost": field_boost
            }
        }
        mlt_queries.append(mlt_query)
    return mlt_queries


def _create_owners_filter(teams, user):
    return {'bool': {
        'should': [
            {
                'bool': {
                    'must': [
                        {'terms': {'owners': teams}},
                        {'exists': {'field': 'owners'}}
                    ]
                }
            },
            {
                'bool': {
                    'must': [
                        {'terms': {'userOwners': [user]}},
                        {'exists': {'field': 'userOwners'}}
                    ]
                }
            },
            {'bool': {'must_not': [{'exists': {'field': 'owners'}}]}}
        ]}
    }


def _create_filters(teams, from_date, custom_filters, user):
    filters = []
    if teams:
        teams_filter = _create_owners_filter(teams, user)
        filters.append(teams_filter)
    if custom_filters:
        custom_filters = filter.parse_filter_query(custom_filters)
        filters.append(custom_filters)

    return filters


def _create_aggregations(size, aggs_fields):
    aggs_query = {
        "sample": {
            "sampler": {
                "shard_size": size
            },
            "aggs": {
            }
        }
    }
    for field in aggs_fields:
        aggs_query['sample']['aggs'][field] = {
            "terms": {
                "field": field
            }
        }
    return aggs_query


def create_prediction_query(query_parameters):
    fields = query_parameters.get('fields')
    if not fields:
        fields = [{'name': 'displayName', 'weight': 1}]
    index = query_parameters['index']
    aggs = query_parameters.get('aggs')
    id = query_parameters.get('id')
    doc = query_parameters.get('doc')
    teams = query_parameters.get('teams')
    user = query_parameters.get('current_user')
    from_date = query_parameters['from_date'] if 'from_date' in query_parameters else int(time.time() - 1186400) * 1000
    filter = query_parameters.get('filter')
    output_fields = query_parameters.get('output_fields')
    query = {
        "query": {
            "bool": {
                "should": _create_mlt_part(fields, index, id, doc),
                "filter": _create_filters(teams, from_date, filter, user)
            }
        },
        "sort": [
            {"_score": "desc"},
            {"createDate": {"order": "desc", "unmapped_type": "long"}}
        ],
        "min_score": 0.001
    }
    if output_fields:
        query['_source'] = output_fields

    if aggs:
        size = query_parameters.get('size', 20)
        query['aggs'] = _create_aggregations(size, aggs)
    else:
        size = query_parameters.get('size', 5)
    query['size'] = size

    return query
