""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from datetime import datetime, timedelta

from util.logger import get_logger
from dateutil.relativedelta import relativedelta
import calendar

logger = get_logger('__name__')

UI_FILTER_DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.000Z'
ELASTIC_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'


def getRelativeDate(years=0, months=0, days=0, hours=0, minutes=0, seconds=0):
    current_date = datetime.now()
    relative_date = current_date + relativedelta(years=years)

    if type(months) == str:
        if months == 'start':
            relative_date = relative_date.replace(month=1)
        elif months == 'end':
            relative_date = relative_date.replace(month=12)
        months = 0

    if type(days) == str:
        if days == 'start':
            relative_date = relative_date.replace(day=1)
        elif days == 'end':
            relative_date = relative_date.replace(day=calendar.monthrange(datetime.now().year, datetime.now().month)[1])
        days = 0

    if type(hours) == str:
        if hours == 'start':
            relative_date = relative_date.replace(hour=0)
        elif hours == 'end':
            relative_date = relative_date.replace(hour=23)
        hours = 0

    if type(minutes) == str:
        if minutes == 'start':
            relative_date = relative_date.replace(minute=0)
        elif minutes == 'end':
            relative_date = relative_date.replace(minute=59)
        minutes = 0

    if type(seconds) == str:
        if seconds == 'start':
            relative_date = relative_date.replace(second=0)
        elif seconds == 'end':
            relative_date = relative_date.replace(second=59)
        seconds = 0

    relative_date = relative_date + relativedelta(
        months=months,
        days=days,
        hours=hours,
        minutes=minutes,
        seconds=seconds
    )

    return relative_date.strftime(ELASTIC_DATE_FORMAT)


def parse_relative_date(relative_date):
    relative_date_parts = relative_date.replace("{{getRelativeDate(", "").replace(")}}", "").split(",")
    years, months, days, hours, minutes, seconds = [float(date_part) if date_part not in ['"start"', '"end"', '"start-1"'] else date_part.strip('"')
                                                    for date_part in relative_date_parts]

    return years, months, days, hours, minutes, seconds


def create_date_range_filter(filter, field, operator, value):
    date_filters = filter.get('filters')
    if not date_filters:
        # Case when static date is defined in filter
        date = datetime.strptime(value, UI_FILTER_DATE_FORMAT).strftime(ELASTIC_DATE_FORMAT)
        return {
            "range": {
                field: {
                    operator: date
                }
            }
        }
    else:
        _value = filter.get('_value')
        _field = filter.get('_field')

        # Case when Relative or Custom date is defined in filter
        if len(date_filters) == 1:
            date_filter = date_filters[0]
            _operator = date_filter.get('operator')
            years, months, days, hours, minutes, seconds = parse_relative_date(date_filter['value'])
            date = getRelativeDate(years, months, days, hours, minutes, seconds)

            return {
                "range": {
                    _field: {
                        _operator: date
                    }
                }
            }
        else:
            date_filter1 = date_filters[0]
            _operator1 = date_filter1.get('operator')
            years, months, days, hours, minutes, seconds = parse_relative_date(date_filter1['value'])
            date1 = getRelativeDate(years, months, days, hours, minutes, seconds)

            date_filter2 = date_filters[1]
            _operator2 = date_filter2.get('operator')
            years, months, days, hours, minutes, seconds = parse_relative_date(date_filter2['value'])
            date2 = getRelativeDate(years, months, days, hours, minutes, seconds)

            return {
                "range": {
                    _field: {
                        _operator1: date1,
                        _operator2: date2
                    }
                }
            }

def create_filter(filter):
    field = filter.get('field')
    type = filter.get('type')
    operator = filter.get('operator')
    _operator = filter.get('_operator')
    value = filter.get('value')

    # Handle datetime filter
    if type == 'datetime':
        if operator == 'eq':
            return {
                "term": {
                    field: {
                        "value": datetime.strptime(value, UI_FILTER_DATE_FORMAT).strftime(ELASTIC_DATE_FORMAT)
                    }
                }
            }
        elif operator == 'neq':
            return {
                "bool": {
                    "must_not": {
                        "term": {
                            field: datetime.strptime(value, UI_FILTER_DATE_FORMAT).strftime(ELASTIC_DATE_FORMAT)
                        }
                    }
                }
            }
        elif not operator:
            return create_date_range_filter(filter, field, operator, value)

    # Handle number range filter
    elif _operator in ['lt', 'lte', 'gt', 'gte']:
        return {
                    "range" : {
                        field : {
                            _operator : filter.get('value')
                        }
                    }
                }
    elif _operator == 'eq' and type == 'primitive':
        return {
            "term": {
                field: {
                    "value": value
                }
            }
        }
    elif _operator == 'neq' and type == 'primitive':
        return {
            "bool": {
                "must_not": {
                    "term": {
                        field: value
                    }
                }
            }
        }
    # Handle picklist equality filter
    elif operator == 'eq' and type == 'object' and value:
        return {
                "term": {
                    field: {
                        "value": value
                    }
                }
            }
    elif operator == 'neq' and ( _operator =='neq' or type == 'object'):
        return {
                "bool" : {
                  "must_not" : {
                    "term" : {
                      field : value
                    }
                  }
                }
              }
    elif operator == 'isnull' and value == 'true':
        return {
                "bool": {
                    "must_not": {
                        "exists": {
                            "field": field
                        }
                    }
                }
            }
    elif operator == 'isnull' and (value == 'false' or not value):
        return {
                "exists": {
                    "field": field
            }
        }

    # Handle "Matches Pattern" filter
    elif _operator == 'like_pattern':
        return {
                "wildcard": {
                    field: {
                        "value": value
                    }
                }
            }

    # Handle "Contains" filter
    elif _operator == 'like' and value and value.startswith('%') and value.endswith('%'):
        return {
            "query_string": {
                "default_field": field,
                "query": '*'+value[1:-1]+'*',
                "default_operator": "AND"
            }
        }
    # Handle "Not Contains" filter
    elif _operator == 'notlike' and value and value.startswith('%') and value.endswith('%'):
        return {
            "bool": {
                "must_not": [{
                    "wildcard": {
                        field: {
                            "value": '*'+value[1:-1]+'*'
                        }
                    }
                }]
            }
        }

    elif operator == 'in':
        module = filter.get('module')
        if module == 'recordTags':
            # tags search come as `/api/3/tags/Tag while tag is indexed as Tag
            actual_tags = []
            for tag_iri in value:
                tag = tag_iri.replace('/api/3/tags/', '')
                actual_tags.append(tag)
            value = actual_tags
        return {
            "terms": {
                field: value
            }
        }
    else:
        logger.warning('unknown operator '+str(operator))
    return {}


def parse_filter_query(filter_query):
    if filter_query.get('logic') in ['AND', 'OR'] and filter_query.get('type') != 'datetime':
        filters = filter_query['filters']
        query_type = 'must' if filter_query['logic'] == 'AND' else 'should'
        bool_query = {
            'bool' : {
                query_type: []
            }
        }
        for filter in filters:
            parsed_filter = parse_filter_query(filter)
            if parsed_filter:
                bool_query['bool'][query_type].append(parsed_filter)
        return bool_query
    else:
        return create_filter(filter_query)