""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from constants import API_PREFIX
from util.communicator import _make_request
from util.config import config
from constants import IGNORE_REVERSE_INDEXING_FOR_FORM_TYPE


def get_cyops_mmd():
    url = "https://" + config.get('crudhub', 'host') + \
          API_PREFIX + 'model_metadatas' + "?$export=true&$limit=2147483647&$orderby=type&$relationships=true"
    headers = {'Content-Type': 'application/json'}
    api_data = _make_request('get', url, body=None, headers=headers, allow_503=False)
    if api_data.get('hydra:totalItems'):
        return api_data.get("hydra:member")
    else:
        return []


def get_encrypted_searchable_fields(attribute):
    encrypted_fields = []
    datetime_fields = []
    json_field =[]
    file_field =[]
    ignore_reverse_index_module = []
    for item in attribute:
        if item.get('encrypted'):
            encrypted_fields.append(item.get('name'))
        elif item.get('formType') == 'datetime':
            datetime_fields.append(item.get('name'))
        elif item.get('formType') == 'file':
            file_field.append(item.get('name'))
        elif item.get('type') == 'object' or item.get('formType') == 'object':
            json_field.append(item.get('name'))
        elif item.get('formType') in IGNORE_REVERSE_INDEXING_FOR_FORM_TYPE:
            ignore_reverse_index_module.append(item.get('name'))
    return encrypted_fields, datetime_fields, json_field, file_field


def get_mmd_meta():
    system_module_meta = {'display_name': '{{ name }}', 'datetime_fields': ['modifyDate', 'createDate']}
    cyops_mmd = get_cyops_mmd()
    modules_mmd = {}
    for module in cyops_mmd:
        fields_list, datetime_list, json_list ,file_field = get_encrypted_searchable_fields(module['attributes'])
        peer_replicable = module.get('peerReplicable', False)
        if module.get('trackable'):
            datetime_list.append('modifyDate')
            datetime_list.append('createDate')
        mod = {'display_name': module['displayName'],
               'encrypted_fields': fields_list,
               'datetime_fields': datetime_list,
               'peer_replicable': peer_replicable,
               'json_field': json_list,
               'file_field': file_field}
        modules_mmd[module['type']] = mod

    modules_mmd['workflows'] = system_module_meta
    modules_mmd['rules'] = system_module_meta
    modules_mmd['workflow_collections'] = system_module_meta
    modules_mmd['rule_collections'] = system_module_meta
    return modules_mmd
