""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import yaml
import configparser

from constants import *


class Config(object):
    config = {}

    def __init__(self):
        for ini_file in CONFIG_FILES_INI:
            self.config.update(Config.read_ini_file(ini_file))
        for properties_file in CONFIG_FILES_PROPERTIES:
            self.config.update(self.read_properties_file(properties_file))
        for yaml_file in CONFIG_FILES_YAML:
            self.config.update(Config.read_yaml_file(yaml_file))

    def read_properties_file(self, properties_file_path):
        config = {}
        separators = ['=', ':', ' ']
        with open(properties_file_path, 'r') as properties_file:
            for line in properties_file:
                strip_line = line.strip()
                if strip_line.startswith('#'):
                    continue
                if '#' in strip_line:
                    strip_line = strip_line.split('#', 1)[0].strip()
                for separator in separators:
                    if separator in strip_line:
                        key_value = strip_line.split(separator, 1)
                        key = key_value[0].strip()
                        value = key_value[1].strip()
                        config[key] = value
                        break
        return config

    @staticmethod
    def read_yaml_file(yaml_file_path):
        config = {}
        with open(yaml_file_path, 'r') as yaml_file:
            config.update(yaml.safe_load(yaml_file))
        return config

    @staticmethod
    def read_ini_file(ini_file):
        config = {}
        parser = configparser.ConfigParser()
        parser.read(ini_file)
        for section in parser.sections():
            config[section] = dict(parser.items(section))
        return config

    def get(self, section, option=None, fallback=None):
        if option:
            try:
                return self.config.get(section, {}).get(option, fallback)
            except Exception as e:
                return fallback
        return self.config.get(section, fallback)


config = Config()
