""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import subprocess
import time
from ssl import create_default_context

import pika
import urllib3
from elasticsearch import RequestsHttpConnection
from elasticsearch_dsl import connections

from util.config import config
from util.logger import get_logger

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

logger = get_logger(__name__)

es_host = config.get('elasticsearch', 'es_host')
es_port = config.get('elasticsearch', 'es_port')
es_user = config.get('elasticsearch', 'es_user')
es_secret = config.get('elasticsearch', 'secret')

if es_secret:
    decrypt_secret = subprocess.check_output(['python3', '/opt/cyops/configs/scripts/manage_passwords.py', '--decrypt',
                                          es_secret, 'jQp3(7@jod#j38d1']
                                         ).strip().decode()
    if not decrypt_secret.startswith('Password:'):
        pass
    else:
        es_secret = str(decrypt_secret[len('Password:'):])

es_use_ssl = config.get('elasticsearch', 'use_ssl')

try:
    if es_use_ssl:
        es_ssl_cert_path = config.get('elasticsearch', 'ssl_cert_path')
        if (es_ssl_cert_path.strip() != ''):
            context = create_default_context(cafile=es_ssl_cert_path)
            es = connections.create_connection(hosts=[es_host], port=es_port, http_auth=(es_user, es_secret),
                                            scheme="https", ssl_context=context, timeout=30)
        else:
            es = connections.create_connection(hosts=[es_host], port=es_port, http_auth=(es_user, es_secret), timeout=30, use_ssl=True, verify_certs=False, connection_class=RequestsHttpConnection, ssl_show_warn=False)
    else:
        es = connections.create_connection(hosts=[es_host], port=es_port, http_auth=(es_user, es_secret), timeout=30)
except Exception:
    logger.exception("Unable to create elastic_search connection")
    time.sleep(10)
    exit(1)


def get_rabbitmq_channel():
    host = config.get('cyops.rabbitmq.host')
    port = config.get('cyops.rabbitmq.port')
    vhost = config.get('cyops.rabbitmq.vhost')
    username = config.get('cyops.rabbitmq.user')
    password = config.get('cyops.rabbitmq.password')
    connection_retry_count = config.get('rabbitmq_connection', 'retry_count')
    connection_retry_delay = config.get('rabbitmq_connection', 'retry_delay')

    credentials = pika.PlainCredentials(username=username, password=password)
    parameters = pika.ConnectionParameters(host=host, port=port,
                                           virtual_host=vhost, credentials=credentials,
                                           connection_attempts=connection_retry_count,
                                           retry_delay=connection_retry_delay)
    connection = pika.BlockingConnection(parameters)

    channel = connection.channel()
    channel.exchange_declare(exchange='cyops.crudhub.datanotify',
                             exchange_type='topic',
                             passive=False,
                             durable=True,
                             auto_delete=False)

    # Declaring Queue
    channel.queue_declare(queue='queue.falcon.data', durable=True, auto_delete=False)
    channel.queue_declare(queue='queue.falcon.system', durable=True, auto_delete=False)
    channel.queue_declare(queue='queue.falcon.status', auto_delete=False)
    channel.queue_declare(queue='queue.falcon.search', auto_delete=False)

    channel.basic_qos(prefetch_count=1)

    # Binding Queue and Exchanges
    channel.queue_bind(queue='queue.falcon.data', exchange='cyops.crudhub.datanotify',
                       routing_key='cyops.crudhub.datanotify')
    channel.queue_bind(queue='queue.falcon.system', exchange='cyops.crudhub.datanotify',
                       routing_key='cyops.crudhub.systemnotify')
    channel.queue_bind(queue='queue.falcon.search', exchange='cyops.crudhub.datanotify',
                       routing_key='cyops.crudhub.essearch')
    channel.queue_bind(queue='queue.falcon.status', exchange='cyops.crudhub.datanotify',
                       routing_key='cyops.crudhub.status')
    return channel
