#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2021 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

tune_logger_setting_for_elasticsearch() {
  local log4j_file=/etc/elasticsearch/log4j2.properties
  local max_size_per_file=10MB
  local max_file_count=10

  # Below is for elasticsearch.log
  # Default behavior:
  # 1. If IfAccumulatedFileSize for all elasticsearch.logs exceeds 2GB, then rotate.
  # New behavior:
  # 1. If IfAccumulatedFileCount exceeds $max_file_count, then rotate. Max size for every elaticsearch.log
  # can be $max_size_per_file.

  sed -i "s/appender.rolling_old.policies.size.size = 128MB/appender.rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file
  sed -i "s/appender.rolling_old.strategy.action.condition.nested_condition.type = IfAccumulatedFileSize/appender.rolling_old.strategy.action.condition.nested_condition.type = IfAccumulatedFileCount/g" $log4j_file
  sed -i "s/appender.rolling_old.strategy.action.condition.nested_condition.exceeds = 2GB/appender.rolling_old.strategy.action.condition.nested_condition.exceeds = $max_file_count/g" $log4j_file

  # Below is for elasticsearch.json
  # Default behavior:
  # 1. If IfAccumulatedFileSize for all elasticsearch_server.json exceeds 2GB, then rotate.
  # New behavior:
  # 1. If IfAccumulatedFileCount exceeds $max_file_count, then rotate. Max size for every elasticsearch_server.json
  # can be $max_size_per_file.

  sed -i "s/appender.rolling.policies.size.size = 128MB/appender.rolling.policies.size.size = $max_size_per_file/g" $log4j_file
  sed -i "s/appender.rolling.strategy.action.condition.nested_condition.type = IfAccumulatedFileSize/appender.rolling.strategy.action.condition.nested_condition.type = IfAccumulatedFileCount/g" $log4j_file
  sed -i "s/appender.rolling.strategy.action.condition.nested_condition.exceeds = 2GB/appender.rolling.strategy.action.condition.nested_condition.exceeds = $max_file_count/g" $log4j_file

  # Below is for elasticsearch_deprecation.json
  # Default behavior:
  # 1. Rotate elasticsearch_deprecation.json if file size exceeds 1GB.
  # 2. Total files can be 4.
  # New behavior:
  # 1. Rotate elasticsearch_deprecation.json if file size exceeds $max_size_per_file.
  # 2. Total files can be 4. (no change)

  sed -i "s/appender.deprecation_rolling.policies.size.size = 1GB/appender.deprecation_rolling.policies.size.size = $max_size_per_file/g" $log4j_file

  # Below is for elasticsearch_deprecation.log
  # Default behavior:
  # 1. Rotate elasticsearch_deprecation.log if file size exceeds 1GB.
  # 2. Total files can be 4.
  # New behavior:
  # 1. Rotate elasticsearch_deprecation.log if file size exceeds $max_size_per_file.
  # 2. Total files can be 4. (no change)

  sed -i "s/appender.deprecation_rolling_old.policies.size.size = 1GB/appender.deprecation_rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file

  # Below is for elasticsearch_index_search_slowlog.json
  # Default behavior:
  # 1. Rotate elasticsearch_index_search_slowlog.json if file size exceeds 1GB.
  # 2. Total files can be 4.
  # New behavior:
  # 1. Rotate elasticsearch_index_search_slowlog.json if file size exceeds $max_size_per_file.
  # 2. Total files can be 4. (no change)

  sed -i "s/appender.index_search_slowlog_rolling.policies.size.size = 1GB/appender.index_search_slowlog_rolling.policies.size.size = $max_size_per_file/g" $log4j_file

  # Below is for elasticsearch__index_search_slowlog.log
  # Default behavior:
  # 1. Rotate elasticsearch_index_search_slowlog.log if file size exceeds 1GB.
  # 2. Total files can be 4.
  # New behavior:
  # 1. Rotate elasticsearch_index_search_slowlog.log if file size exceeds $max_size_per_file.
  # 2. Total files can be 4. (no change)

  sed -i "s/appender.index_search_slowlog_rolling_old.policies.size.size = 1GB/appender.index_search_slowlog_rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file

  # Below is for elasticsearch_index_indexing_slowlog.json
  # Default behavior:
  # 1. Rotate elasticsearch_index_indexing_slowlog.json if file size exceeds 1GB.
  # 2. Total files can be 4.
  # New behavior:
  # 1. Rotate elasticsearch_index_indexing_slowlog.json if file size exceeds $max_size_per_file.
  # 2. Total files can be 4. (no change)

  sed -i "s/appender.index_indexing_slowlog_rolling.policies.size.size = 1GB/appender.index_indexing_slowlog_rolling.policies.size.size = $max_size_per_file/g" $log4j_file

  # Below is for elasticsearch_index_indexing_slowlog.log
  # Default behavior:
  # 1. Rotate elasticsearch_index_indexing_slowlog.log if file size exceeds 1GB.
  # 2. Total files can be 4.
  # New behavior:
  # 1. Rotate elasticsearch_index_indexing_slowlog.log if file size exceeds $max_size_per_file.
  # 2. Total files can be 4. (no change)

  sed -i "s/appender.index_indexing_slowlog_rolling_old.policies.size.size = 1GB/appender.index_indexing_slowlog_rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file

}

set_java_home_globally()
{
   local system_java_home_path="$java_home"
   local global_environment_file=/etc/environment
   if grep -qE "^\s*JAVA_HOME=" $global_environment_file; then
        #JAVA_HOME is already set, replace old line
        #Replace forward slash(/) with \/ in path
        system_java_home_path=`echo $system_java_home_path | sed  -e 's:/:\\\/:g'`
        sed -i "s/^\s*JAVA_HOME=.*$/JAVA_HOME=$system_java_home_path/g" $global_environment_file
        return
    fi
    echo "ES_JAVA_HOME=$system_java_home_path" >> $global_environment_file
}

set_java_home_for_elasticsearch()
{
    local system_java_home_path="$java_home"
    local es_conf_file=/etc/sysconfig/elasticsearch
    if grep -qE "^\s*JAVA_HOME=" $es_conf_file; then
        #JAVA_HOME is already set, replace old line
        #Replace forward slash(/) with \/ in path
        system_java_home_path=`echo $system_java_home_path | sed  -e 's:/:\\\/:g'`
        sed -i "s/^\s*JAVA_HOME=.*$/JAVA_HOME=$system_java_home_path/g" $es_conf_file
    else
        #Add JAVA_HOME
        echo "ES_JAVA_HOME=$system_java_home_path" >> $es_conf_file
    fi
    set_java_home_globally
}

d_install_log=$1
f_install_log=$2

{
  f_tmp_location=/usr/share/elasticsearch/tmp
  es_conf_file=/etc/sysconfig/elasticsearch
  global_environment_file=/etc/environment
  mkdir -p $f_tmp_location
  chown -R elasticsearch:elasticsearch $f_tmp_location
  chmod -R 755 $f_tmp_location

  if grep -Fxq "ES_TMPDIR=$f_tmp_location" "$es_conf_file"; then
    echo "ES_TMPDIR already set"
  else
      echo "ES_TMPDIR=$f_tmp_location" >> $es_conf_file
      echo "ES_TMPDIR=$f_tmp_location" >> $global_environment_file
  fi


  yes | cp /opt/cyops/configs/cyops-search/security.policy /etc/elasticsearch/security.policy
  yes | cp /opt/cyops/configs/cyops-search/fsr.options /etc/elasticsearch/jvm.options.d/fsr.options
  java_home=/usr
  chown elasticsearch:elasticsearch /etc/elasticsearch/security.policy
  chown elasticsearch:elasticsearch /etc/elasticsearch/jvm.options.d/fsr.options
  set_java_home_for_elasticsearch
  tune_logger_setting_for_elasticsearch
  . $es_conf_file
  . $global_environment_file
  chown elasticsearch:elasticsearch -R /etc/elasticsearch/
} >> ${d_install_log}/${f_install_log} 2>&1
