# Copyright start
# Copyright (C) 2008 - 2021 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

i_password=$1
i_password_encrypted=$2
install_type=$3

f_config=/opt/cyops/configs/database/db_config.yml
f_default_config=/opt/cyops/configs/database/db_config.yml.default
f_conf_util=/opt/cyops/configs/scripts/confUtil.py

set_default_user() {
  # set default user passwords
  if $(/usr/share/elasticsearch/bin/elasticsearch-keystore list | grep -q 'bootstrap.password'); then
    echo "bootstrap password already set"
  else
    echo "setting default users..."
    printf $i_password | /usr/share/elasticsearch/bin/elasticsearch-keystore add "bootstrap.password"
  fi
}

update_es_certs() {
  if [ ! -f /etc/elasticsearch/server.leaf.crt ]; then
    echo "updating certs"
    yes | cp /etc/pki/cyops/server.leaf.crt /etc/elasticsearch/server.leaf.crt
    yes | cp /etc/pki/cyops/server.leaf.key /etc/elasticsearch/server.leaf.key
    chmod g=u /etc/elasticsearch/server.leaf.*
  fi
}

enable_es_security() {
  # enable security
  if grep -Fxq "xpack.security.enabled:  true" "$f_elasticsearch_xml"; then
    echo "security is already enabled for elasticsearch"
  else
    echo "enabling security for elasticsearch"
    echo "xpack.security.enabled:  true" >>"$f_elasticsearch_xml"
  fi

  if grep -Fxq "xpack.security.http.ssl.enabled: true" "$f_elasticsearch_xml"; then
    echo "ssl is already enabled for elasticsearch"
  else
    echo "enabling ssl for elasticsearch"
    # https://github.com/koalaman/shellcheck/wiki/SC2129
    {
      echo "xpack.security.http.ssl.enabled: true"
      echo "xpack.security.http.ssl.key: /etc/elasticsearch/server.leaf.key"
      echo "xpack.security.http.ssl.certificate: /etc/elasticsearch/server.leaf.crt"
    } >> "$f_elasticsearch_xml"

  fi
}

enable_external_connection() {
  if grep -Fxq "network.host: 0.0.0.0" "$f_elasticsearch_xml"; then
    echo "external connection configured"
  else
    echo "Enabling ElasticSearch For External Connections"
    sed -i 's/#network.host: 192.168.0.1/network.host: 0.0.0.0/g' "$f_elasticsearch_xml"
    echo "Enabling discovery type as single node"
    sed -i '/#http.port: 9200/a discovery.type: single-node' "$f_elasticsearch_xml"

    echo "Changing log path in jvm.options"
    sed -i 's/logs\//\/var\/log\/elasticsearch\//g' $d_elasticsearch_base/jvm.options
  fi
}


# Not require the ingest.geoip.downloader.enabled feature and save some resources consumption
disable_geoip(){
    local s_geoip="ingest.geoip.downloader.enabled"
    local s_state=$(grep -w "$s_geoip" $f_elasticsearch_xml | awk {'print $2'})
    if [ ! -z "$s_state" ] && [ "$s_state" = "false" ]; then
        #
        # In restore the db sub-command runs the setup_elastic.sh file with input "upgrade", etc, parameters in \
        # restore_password_es function. So disable_geoip function runs and \
        # avoid to make duplicate entry of "ingest.geoip.downloader.enabled" \
        # in /etc/elasticsearch/elasticsearch.yml file.
        #
        return
    fi
    if [ ! -z "$s_state" ] && [ "$s_state" != "false" ]; then
        # c in sed command: Replace complete line
        sed -i "/$s_geoip/c$s_geoip:\ false" $f_elasticsearch_xml
        return
    fi
    echo "$s_geoip: false" >> $f_elasticsearch_xml
}

modify_elastic_config() {
  config_file=$1
  es_user_value=$(python3 $f_conf_util -f $config_file -g es_user)
  if [ "$es_user_value" = "None" ]; then
    python3 $f_conf_util -f "$config_file" -k es_user -v elastic
    python3 $f_conf_util -f "$config_file" -k secret -v "$i_password_encrypted"
    python3 $f_conf_util -f "$config_file" -k use_ssl -v true
  fi
}

d_elasticsearch_base="/etc/elasticsearch"
f_elasticsearch_xml="/etc/elasticsearch/elasticsearch.yml"

if [ "$install_type" = "fresh" ]; then
  echo "This is fresh install. Updating db_config.yml file"
  set_default_user
  update_es_certs
  enable_es_security
  enable_external_connection
  modify_elastic_config $f_config
  disable_geoip
fi

if [ "$install_type" = "upgrade" ]; then
  node_id=$(csadm license --get-device-uuid)
  csadm ha list-nodes --primary | grep -q "$node_id" &>/dev/null
  if [ $? == 0 ]; then
    echo "This is primary node. Updating db_config.yml file"
    modify_elastic_config $f_config

    echo "Propagating es password change to secondary nodes, if any"
    /bin/hamanager add-command --async --commandId 'update_es_credentials' --command "/opt/cyops-auth/utilities/ha/cmd_executor.py --command update-es-credentials --input-arg $i_password_encrypted"
  else
    echo "This is secondary node. updating db_config.yml.default file"
    modify_elastic_config $f_default_config
  fi
  disable_geoip
fi


python3 $f_conf_util -f $f_config -k use_ssl -v true
